----[[
--
-- ---===:::{{{ A U L E ' S    T H E   M A G I C   W O R D }}}:::===---
--
--
-- Author		: joeh@foldingrain.com
-- Version		: 0.5a
-- Release Date	: Sep 26, 2006
-- Main			: Aule [Warlock]
-- Server		: Dunemaul
-- Guild		: NoX (Best Guild Ever)
-- Testers		: Janadine, Kaed, Zerobane, Houseofm
--
--
-- Copyright (c)2006.  All Rights Reserved.
-- All players of World of Warcraft have permission to copy, reproduce, 
-- distribute and modify this computer code in any way they wish with no 
-- permission from the author or any corporation or organization.  The
-- author respectfully requests he be credited for the original inception.
--
--
-- OVERVIEW:
-- ---------
-- The Magic Word is a DKP utility with the following main features:
--
--	- The user (the guild DKP manager) brings up The Magic Word during 
--	  looting.
--	- The user hits the "broadcast magic word" button which tells the
--	  raid what the magic word is and begins a countdown.
--	- All users who whisper the magic word to the user appear in the
--	  list, so it is extremely easy for the user to separate users who
--	  wish an item from the rest of chat.
--	- The Magic Word reports when bidding is closed, and the user can
--	  select one of the users to broadcast as the winner of the item.
--
--
-- DEVELOPERS:
-- -----------
--
--
-- CREDITS:
-- --------
-- Thanks to Scrum and Justin Milligan for their work on CountDoom,
-- which taught me just about everything I needed to know about
-- dealing with curses in an AddOn.
--
--
-- KNOWN ISSUES:
-- -------------
--	- None.
--
--
-- TO DO:
-- ------
--
--
-- RELEASES:
-- ---------
-- Version 0.5a:
--	- Users can now chat while the interface is up.
--	- The user can now click the item link in the interface to see the item details.
--	- Now gathers item details from whipser and raid warning.
--
-- Version 0.4a:
--	- Now collects whipsers the entie time the interface is showing.
--	- Self whispers are no longer stored in the responder list.
--	- Item link is now announced in Raid Warning.
--	- Window is now draggable with the mouse.
--	- When the user clicks the item, the link is whipsered to them so they can click it
--	  and get item information.
--	- Got rid of an unneccessary frame.
--	- Implemented a button to clear responders list.
--
-- Version 0.3a:
--	- Proper implementation of the scroll frame.
--	- Added the ability to suggest which classes should bid on an item.
--	- Added the ability to set an item link by whipsering an item to the DKPM.
-- 
-- Version 0.2a:
--	- Give the raid a set of warning message before the actual countdown begins.
--	- Respond to tells, confirming that their request was received.
--	- Only allow members to bid once.  Don't record dupes.
--	- Sort the scroll frame entries.
--
-- Version 0.1a:
--	- Initial alpha.
--
--]]


-- ----------------------------------------------------------------------------
-- ----------------------------------------------------------------------------
-- P E R S I S T A N T   S E T T I N G S
-- ----------------------------------------------------------------------------
-- ----------------------------------------------------------------------------
Default_TheMagicWord_Config = {
	Version = "0.04",
	TheMagicWord = "ME",
	CountdownDuration = 15
};


-- ----------------------------------------------------------------------------
-- ----------------------------------------------------------------------------
-- C O N S T A N T S
-- ----------------------------------------------------------------------------
-- ----------------------------------------------------------------------------

-- ----- DEPLOY MODE -----
--
-- Sets the deploy mode for the application.  Set to:
--
-- "DEV" for development.
--	- Debugging is off by default.
--	- Say-Solo is off by default.
--
-- "RELEASE" for standard release.
--	- Debugging is on by default.
--	- Say-Solo is on by default.
--
local UB_DEPLOY_MODE = "RELEASE";
--local UB_DEPLOY_MODE = "DEV";

RESPONDER_SCROLL_FRAME_LINES = 20;

LINK_METHOD_WHISPER = "LINK_METHOD_WHISPER";
LINK_METHOD_RAID_WARNING = "LINK_METHOD_RAID_WARNING";

-- ----------------------------------------------------------------------------
-- ----------------------------------------------------------------------------
-- G L O B A L S
-- ----------------------------------------------------------------------------
-- ----------------------------------------------------------------------------

-- Only enable this AddOn if the player is a Warlock
local gEnabled = false;

-- indicates whether the AddOn is initialized.
local gInitialized = false;

-- Player's name
local gPlayerName = nil;

-- The current channel to announce messages on.  Currently one of "SAY", 
-- "PARTY", or "RAID".
local gAnnounceChannel = "NONE";

-- Indicates whether or not to spit out debug messages.
local gDebugFlag = true;

-- Indicates whether the interface is showing or not
local gUIShowing = false;

-- Countdown timer.
local gCountdownTimer = 0;
local gCountingDown = false;
local gSimTime = 0;
local gCountdownStartTime = 0;
local gRealCountdownDuration = 0;

-- Update tracker
local gTimeSinceLastUpdate = 0;

local gResponderArray = {};
local gResponderIndex = 1;
local gResponderLookup = {};
gItemLinkMethod = LINK_METHOD_RAID_WARNING;
gItemUpForBid = "";
gItemLink = nil;


-- ----------------------------------------------------------------------------
-- ----------------------------------------------------------------------------
-- F U N C T I O N S
-- ----------------------------------------------------------------------------
-- ----------------------------------------------------------------------------

--[[
--
-- FUNCTION
--
-- Prints 'msg' if and only if the output frame is valid.
--
--]]
function TheMagicWord_Print(msg)
	local msgOutput = DEFAULT_CHAT_FRAME;
	if (msgOutput) then
		msgOutput:AddMessage("<TheMagicWord> " .. msg, 1, .25, 1);
	end
end


--[[
--
-- FUNCTION
--
-- Prints 'msg' if and only if the output frame is valid and the debug flag 
-- is up.
--
--]]
function TheMagicWord_Debug(msg)
	local msgOutput = DEFAULT_CHAT_FRAME;
	if (msgOutput and gDebugFlag) then
		msgOutput:AddMessage("<TheMagicWord> " .. msg, 1, .25, 1);
	end
end


--[[
--
-- FUNCTION
--
-- Sends a chat message on the selected channel.
--
--]]
function TheMagicWord_Say(msg)
	
	-- If the player is solo and they have say-when-solo off, don't display the message
	if (gAnnounceChannel == "SAY" and TheMagicWord_Config.SayWhenSolo == false) then
		return;
	end
	SendChatMessage(msg, gAnnounceChannel);

end

function TheMagicWord_Whisper(msg, person)
	SendChatMessage("<TheMagicWordBC> " .. msg, "WHISPER", nil, person);
end

function TheMagicWord_RaidWarning(msg)
	
	SendChatMessage(msg, "RAID_WARNING");

end

--[[
--
-- FUNCTION
--
-- Prints 'msg' if and only if the output frame is valid and the debug flag 
-- is up.
--
--]]
TheMagicWord_RegisterCommands = function()
	SLASH_THEMAGICWORD1 = "/tmw";
	SlashCmdList["THEMAGICWORD"] = function(msg)
		TheMagicWord_Slash(msg);
	end
end;


--[[
--
-- FUNCTION
--
-- Processes slash commands.
--
--]]
function TheMagicWord_Slash(msg)
	TheMagicWord_Debug("Slash command: " .. msg);

	msg = string.lower(msg);
	local first, last, cmd, val = string.find (msg, "(%w+) ([_%w]+)");

	if (cmd == nil and val == nil) then
		first, last, cmd = string.find(msg, "(%w+)");
	end

	if (cmd == nil) then
		TheMagicWordUIFrame:Show();
		--TheMagicWord_DisplayCommands();
		return;
	end


	-- UI Panel
	if (cmd == "" or cmd == "config") then
		TheMagicWordUIFrame:Show();
	end


	-- Enable / disable
	if (cmd == "enable") then

		if (gEnabled == true) then 
			return; 
		else
			gEnabled = true;
			TheMagicWord_Initialize();
		end

	elseif (cmd == "disable") then

		if (gEnabled == true) then
			return;
		else
			gEnabled = false;
			unregisterEvents();
		end

	end


	-- Debugging
	if (cmd == "debug") then
		if (val == nil) then
			TheMagicWord_Print("Debugging: " .. tostring(gDebugFlag));
		else
			if (val == "on") then
				gDebugFlag = true;
				TheMagicWord_Print("Debugging is now on.");
			elseif (val == "off") then
				gDebugFlag = false;
				TheMagicWord_Print("Debugging is now off.");
			else
				TheMagicWord_Print("Unable to understand: " .. val);
			end
		end
	end


end


-- check each variable in two tables making sure their variable type match.
function TheMagicWord_TableTypeMatch(table1, table2)
	if (table1 == nil or table2 == nil) then return false; end
	for key,value in pairs(table1) do
		if type(table1[key]) ~= type(table2[key]) then
			return false
		end
	end
	return true;
end


--[[
--
-- FUNCTION
--
-- Displays commands.
--
--]]
TheMagicWord_DisplayCommands = function()
	TheMagicWord_Print("TheMagicWord Usage:");
	TheMagicWord_Print("/ub <enable|disable>         -- enable/disable TheMagicWord" );
	TheMagicWord_Print("/ub debug <on|off>           -- print debug msgs" );
	TheMagicWord_Print("/ub saysolo <on|off>        -- announce timer aloud when solo");
end;


--[[
--
-- FUNCTION
--
-- Registers this AddOn for all base and world events.
--
--]]
function registerEvents()
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
	registerWorldEvents();
end


--[[
--
-- FUNCTION
--
-- Registers this AddOn for world events only.
--
--]]
function registerWorldEvents()

	-- Used to detect when a spell was resisted or immune
	this:RegisterEvent("CHAT_MSG_WHISPER");
	this:RegisterEvent("CHAT_MSG_RAID_WARNING");
	this:RegisterEvent("PARTY_MEMBERS_CHANGED");

end


--[[
--
-- FUNCTION
--
-- Unregisters this AddOn for all base and world events.
--
--]]
function unregisterEvents()
	this:UnregisterEvent("PLAYER_ENTERING_WORLD");
	this:UnregisterEvent("PLAYER_LEAVING_WORLD");
	unregisterWorldEvents();
end


--[[
--
-- FUNCTION
-- 
-- Deregisters this AddOn for world events only.
--
--]]
function unregisterWorldEvents()
	this:UnregisterEvent("CHAT_MSG_WHISPER");
	this:UnregisterEvent("CHAT_MSG_RAID_WARNING");
	this:UnregisterEvent("PARTY_MEMBERS_CHANGED");
end


--[[
--
-- FUNCTION
--
-- 
--
--]]
function TheMagicWord_InitVars()
	-- First install or upgrade, Load Variable from default and show config window
	if (type(TheMagicWord_Config) ~= "table" or 
		not TheMagicWord_TableTypeMatch(Default_TheMagicWord_Config, TheMagicWord_Config) or 
		(TheMagicWord_Config.Version ~= Default_TheMagicWord_Config.Version)) then
		TheMagicWord_Config = {};
		TheMagicWord_Config = Default_TheMagicWord_Config;
		TheMagicWord_Debug("No config set.  Loaded defaults.");
	else
		TheMagicWord_Debug("Config found and loaded.");
	end

	if (Default_TheMagicWord_Config.Version) then
		TheMagicWord_Debug("Config is valid...");
	end
end


--[[
--
-- FUNCTION
--
-- Initializes the AddOn.
--
--]]
function TheMagicWord_Initialize()

	-- Check deploy mode.
	if (UB_DEPLOY_MODE == "DEV") then
		gDebugFlag = true;
	elseif (UB_DEPLOY_MODE == "RELEASE") then
		gDebugFlag = false;
	end


	gEnabled = true;
	gUIShowing = false;

	gPlayerName = UnitName("player");
	
	TheMagicWord_Debug("Player recorded.");
	registerEvents();

	-- Slash interface
	TheMagicWord_RegisterCommands();

	gInitialized = true;
	TheMagicWord_Print(TMW_TITLE_VERSION .. " initialized.");
	TheMagicWord_Print("/tmw To start The Magic Word.");

end


--[[
--
-- FUNCTION
--
-- Returns a group count, automatically determining if we are in a 
-- party or a raid.
--
--]]
function TheMagicWord_GetGroupCount()

	local groupCount = GetNumRaidMembers();
	if (groupCount == 0) then
		-- We are not in a raid.
		groupCount = GetNumPartyMembers();
	end

	return groupCount;

end


--[[
--
-- FUNCTION
--
-- Callback for OnLoad event.
--
--]]
function TheMagicWord_OnLoad()
	gInitialized = false;
	TheMagicWord_Initialize();
end

function extractItemLinkFromChatItemLink(message)
	if (message == nil) then
		return nil;
	end
	local _, _, _, _itemLink = string.find(message, "(|H(.+)|h)");
	if (_itemLink == nil) then
		return nil;
	end
	_, _, _itemLink = string.find(_itemLink, "(.-|h)");
	return _itemLink;
end

--[[
--
-- FUNCTION
--
-- Callback for OnEvent.
--
--]]
function TheMagicWord_OnEvent(event, arg1, arg2)

	--TheMagicWord_Debug("Got An Event!!!" .. event);

	if (event == "CHAT_MSG_WHISPER") then

		-- FOR DEBUG TESTING OF SCROLL FRAME
		--[[gResponderArray[gResponderIndex] = arg1;
		gResponderIndex = gResponderIndex + 1;
		TheMagicWord_UpdateResponderScrollFrame();--]]

		local item = extractItemLinkFromChatItemLink(arg1);
		if (item ~= nil) then
			gItemLinkMethod = LINK_METHOD_WHISPER;
			gItemUpForBid = arg1;
			gItemLink = item;
			TheMagicWordUIFrame_ItemEdit:SetText(gItemUpForBid);
			--[[TheMagicWord_Debug("JOE::::: MESG: " .. arg1);
			TheMagicWord_Say("JOE::::: " .. arg1);
			return;--]]
		end

		-- If the UI isn't showing, don't collect whispers
		if (gUIShowing == false) then
			return;
		end

		-- Don't trap self-whispers
		if (arg2 == gPlayerName) then
			return;
		end

		-- 0.4a:  We're gonna try having it capture whispers the entire time it's showing.
		--[[if (gCountingDown == false) then
			TheMagicWord_Debug("JOE:::: We're NOT counting down!!");
			return;
		end--]]

		-- Has this person already bid???  We still update it, but slap them down.
		local alreadyBid = false;
		if (gResponderLookup[arg2] ~= nil and gCountingDown) then
			TheMagicWord_Whisper("I already got your bid!!  BUG OFF ALREADY!!", arg2);
			alreadyBid = true;
		end

		-- Prepare
		local response = arg2 .. ": " .. arg1;
        local found = string.find(string.upper(arg1), string.upper(TheMagicWord_Config.TheMagicWord));
		local index = gResponderIndex;
		if (alreadyBid) then
			index = gResponderLookup[arg2];
		else
			gResponderIndex = gResponderIndex + 1;
		end

		if (found) then
			gResponderArray[index] = arg2 .. ": " .. arg1 .. " <-- MAGIC WORD!!!";
			gResponderLookup[arg2] = index;
		else
			gResponderArray[index] = arg2 .. ": " .. arg1;
			gResponderLookup[arg2] = index;
		end
		TheMagicWord_UpdateResponderScrollFrame();

		if (alreadyBid == false and gCountingDown) then
			TheMagicWord_Whisper("Received your bid!  Thank you!  Now bug off...", arg2);
		end

	elseif (event == "CHAT_MSG_RAID_WARNING") then

		local item = extractItemLinkFromChatItemLink(arg1);
		if (item ~= nil) then
			gItemLinkMethod = LINK_METHOD_RAID_WARNING;
			gItemUpForBid = arg1;
			gItemLink = item;
			TheMagicWordUIFrame_ItemEdit:SetText(gItemUpForBid);
		end

	elseif (event == "PARTY_MEMBERS_CHANGED") then

		TheMagicWord_UpdateAnnounceChannel();

	elseif event == "PLAYER_ENTERING_WORLD" then

		-- Player has changed zones
		TheMagicWord_InitVars();
		TheMagicWord_UpdateAnnounceChannel();
		registerWorldEvents();

	elseif event == "PLAYER_LEAVING_WORLD" then

		-- Player has changed zones
	    unregisterWorldEvents();

	end

end


--[[
--
-- FUNCTION
--
-- 
--
--]]
function TheMagicWord_UpdateAnnounceChannel()
	-- Are we in a raid, a party, or solo?
	local groupCount = TheMagicWord_GetGroupCount();

	if (groupCount == 0) then
		-- Solo
		gAnnounceChannel = "SAY";
	elseif (groupCount <= 5) then
		-- Party
		gAnnounceChannel = "PARTY";
	elseif (groupCount > 5) then
		-- Raid
		gAnnounceChannel = "RAID";
	end
end


--[[
--
-- FUNCTION
--
-- Callback for the OnUpdate event.
--
--]]
function TheMagicWord_OnUpdate(arg1)
	-- Update all timers.
	gSimTime = gSimTime + arg1;
	TheMagicWord_UpdateCountdownTimer();
end


--[[
--
-- FUNCTION
--
-- 
--
--]]
function TheMagicWord_UpdateCountdownTimer()

	local currentTime = GetTime();

	if (gCountingDown == false) then
		return;
	end

	local rawDelta = currentTime - gCountdownStartTime;
	local tmp = gRealCountdownDuration - floor(rawDelta);
	local intSnap = false;

	-- Report timer only at the integers
	if (tmp < gCountdownTimer) then
		intSnap = true;
		TheMagicWord_Debug("Countdown Timer: " .. tmp);
	end

	gCountdownTimer = tmp;
	
	-- Only report to chat on the second edges.
	if (intSnap) then

		if (gCountdownTimer == gRealCountdownDuration - 1) then

			if (gItemUpForBid ~= nil and string.len(gItemUpForBid) > 0) then
				TheMagicWord_Say("GET READY TO BID: " .. gItemUpForBid);
				if (gItemLinkMethod == ITEM_LINK_WHISPER) then
					TheMagicWord_RaidWarning(gItemUpForBid);
				end
			else
				TheMagicWord_Say("GET READY TO BID!");
			end

		elseif (gCountdownTimer == gRealCountdownDuration - 2) then
			local classes = TheMagicWord_ListClasses();
			if (classes ~= nil and string.len(classes) > 0) then
				TheMagicWord_Say("Classes: " .. classes);
			end
		elseif (gCountdownTimer == gRealCountdownDuration - 3) then
			TheMagicWord_Say("The magic word is: " .. tostring(TheMagicWord_Config.TheMagicWord));
		elseif (gCountdownTimer == gRealCountdownDuration - 4) then
			TheMagicWord_Say("Send your whispers to >>> " .. gPlayerName .. " <<<");
		elseif (gCountdownTimer == gRealCountdownDuration - 5) then
			TheMagicWord_Say("ITEM BIDDING BEGINS NOW!!!");
		elseif (gCountdownTimer > 0) then
			TheMagicWord_Say("COUNTDOWN:  " .. gCountdownTimer);
		end

		if (gCountdownTimer <= tonumber(TheMagicWordUIFrame_CountdownEdit:GetText()) and 
			math.mod(gCountdownTimer, 5) == 0) then
			if (gItemUpForBid ~= nil and string.len(gItemUpForBid) > 0) then
				TheMagicWord_Say("Item:  " .. gItemUpForBid);
			end
		end

	end

	-- The countdown has ended...reset.
	if (gCountdownTimer <= 0) then
		TheMagicWord_Say("Item bidding is over.");
		TheMagicWord_StopCountdown();
	end

end

function TheMagicWord_UpdateResponderScrollFrame()

	local line;
	local offset;

	table.sort(gResponderArray);

	FauxScrollFrame_Update(
		ResponderScrollFrame, 
		table.getn(gResponderArray),
		RESPONDER_SCROLL_FRAME_LINES,
		16);

	for line = 1, RESPONDER_SCROLL_FRAME_LINES, 1 do

		local offset = line + FauxScrollFrame_GetOffset(ResponderScrollFrame);
		local responderItem = getglobal("ResponderItem" .. line);

		if (offset <= table.getn(gResponderArray)) then
			local response = tostring(gResponderArray[offset]);
        	if (string.find(string.upper(response), string.upper(TheMagicWord_Config.TheMagicWord))) then
				responderItem:SetTextColor(1, 1, 0);
			else
				responderItem:SetTextColor(1, 1, 1);
			end
			responderItem:SetText(response);
			responderItem:Show();
		else
			responderItem:Hide();
		end

	end

end

function TheMagicWord_ClearResponderList()
	gResponderArray = {};
	gResponderIndex = 1;
	gResponderLookup = {};
	FauxScrollFrame_Update(
		ResponderScrollFrame, 
		table.getn(gResponderArray),
		RESPONDER_SCROLL_FRAME_LINES,
		16);
	for iItem = 1, RESPONDER_SCROLL_FRAME_LINES, 1 do
		local responder = getglobal("ResponderItem" .. iItem);
		responder:Hide();
	end
end


function TheMagicWord_PopulateUI()

	-- Responder List
	TheMagicWordUIFrame_MagicWordEdit:SetText(TheMagicWord_Config.TheMagicWord);
	TheMagicWordUIFrame_CountdownEdit:SetText(TheMagicWord_Config.CountdownDuration);
	--TheMagicWord_ClearResponderList();

	-- Checkboxes
	CheckButton_Druid:SetChecked(true);
	CheckButton_Hunter:SetChecked(true);
	CheckButton_Mage:SetChecked(true);
	CheckButton_Paladin:SetChecked(true);
	CheckButton_Priest:SetChecked(true);
	CheckButton_Rogue:SetChecked(true);
	CheckButton_Shaman:SetChecked(true);
	CheckButton_Warlock:SetChecked(true);
	CheckButton_Warrior:SetChecked(true);

	-- Item
	local tmp = nil;
	if (gItemUpForBid == nil or string.len(gItemUpForBid) <= 0) then
		tmp = nil;
	else
		tmp = gItemUpForBid;
	end

	if (tmp == nil) then
		TheMagicWordUIFrame_ItemEdit:SetText("-- nothing --");
	else
		TheMagicWordUIFrame_ItemEdit:SetText(tmp);
	end

	-- TESTING
	--[[for i = 1, 50 do
	    gResponderArray[i] = "[" .. math.random(100) .. "] Joe Test " .. i;
	end

	TheMagicWord_UpdateResponderScrollFrame();--]]

end


function TheMagicWord_StartCountdown()

	-- Simple validation
	if (tonumber(TheMagicWordUIFrame_CountdownEdit:GetText()) == nil) then
		TheMagicWordUIFrame_CountdownEdit:SetText(TheMagicWord_Config.CountdownDuration);
	end

	TheMagicWord_ClearResponderList();
	gCountdownStartTime = GetTime();

	-- Extra seconds for pre-countdown messages
	gRealCountdownDuration = tonumber(TheMagicWordUIFrame_CountdownEdit:GetText()) + 6;

	TheMagicWord_Debug("JOE:::: gRealCountdownDuration: " .. gRealCountdownDuration);
	gCountdownTimer = gRealCountdownDuration;
	gCountingDown = true;
	TheMagicWord_CommitConfigSettings();
end

function TheMagicWord_AbortCountdown()
	if (gCountingDown) then
		TheMagicWord_Say(">>> COUNTDOWN ABORTED EARLY!! <<<");
		TheMagicWord_StopCountdown();
	end
end

function TheMagicWord_StopCountdown()
	gCountingDown = false;
	gCountdownTimer = 0;
end

function TheMagicWord_SortResponderList()
	table.sort(gResponderArray);
	TheMagicWord_UpdateResponderScrollFrame();
end

function TheMagicWord_SelectAllClasses(value)
	CheckButton_Druid:SetChecked(value);
	CheckButton_Hunter:SetChecked(value);
	CheckButton_Mage:SetChecked(value);
	CheckButton_Paladin:SetChecked(value);
	CheckButton_Priest:SetChecked(value);
	CheckButton_Rogue:SetChecked(value);
	CheckButton_Shaman:SetChecked(value);
	CheckButton_Warlock:SetChecked(value);
	CheckButton_Warrior:SetChecked(value);
end

function TheMagicWord_ListClasses()

	local result = "";

	if (CheckButton_Druid:GetChecked()) then
		result = result .. "Druids, ";
	end
	if (CheckButton_Hunter:GetChecked()) then
		result = result .. "Hunters, ";
	end
	if (CheckButton_Mage:GetChecked()) then
		result = result .. "Mages, ";
	end
	if (CheckButton_Paladin:GetChecked()) then
		result = result .. "Paladins, ";
	end
	if (CheckButton_Priest:GetChecked()) then
		result = result .. "Priests, ";
	end
	if (CheckButton_Rogue:GetChecked()) then
		result = result .. "Rogues, ";
	end
	if (CheckButton_Shaman:GetChecked()) then
		result = result .. "Shamen, ";
	end
	if (CheckButton_Warlock:GetChecked()) then
		result = result .. "Warlocks, ";
	end
	if (CheckButton_Warrior:GetChecked()) then
		result = result .. "Warriors, ";
	end

	local len = string.len(result);
	if (len > 0) then
		result = string.sub(result, 0, len-2);
	end

	return result;

end

function TheMagicWord_ClearItem()
	gItemUpForBid = "";
	TheMagicWordUIFrame_ItemEdit:SetText("-- nothing --");
end

--[[
--
-- FUNCTION
-- 
-- You *must* set these values explicitly as booleans otherwise they
-- don't persist properly in SavedVariables.
--
--]]
function TheMagicWord_CommitConfigSettings()

	-- The Magic Word
	TheMagicWord_Config.TheMagicWord = tostring(TheMagicWordUIFrame_MagicWordEdit:GetText());

	-- Countdown
	local tmp = tonumber(TheMagicWordUIFrame_CountdownEdit:GetText());
	if (tmp == nil) then
		tmp = Default_TheMagicWord_Config.CountdownDuration;
	end
	TheMagicWord_Config.CountdownDuration = tmp;

	TheMagicWord_Debug("Commit: Magic Word: " .. tostring(TheMagicWord_Config.TheMagicWord));
	TheMagicWord_Debug("Commit: Countdown Duration: " .. tostring(TheMagicWord_Config.CountdownDuration));
end

function TheMagicWord_SetUIShowing(value)
	gUIShowing = value;
end


--[[
--
-- FUNCTION
-- 
-- Currently unused.
--
--]]
function TheMagicWord_OnDragStart()
	this:StartMoving()
end


--[[
--
-- FUNCTION
-- 
-- Currently unused.
--
--]]
function TheMagicWord_OnDragStop()
	this:StopMovingOrSizing()
end


function TheMagicWord_OnItemLinkClick()
	TheMagicWord_Debug("JOE:::::  Item....");
	if (gItemUpForBid == nil or string.len(gItemUpForBid) <= 0) then
		TheMagicWord_Debug("JOE:::::  Not valid!!");
		return;
	end
	SetItemRef(gItemLink, gItemUpForBid, "LeftButton");
end
